#-----------------------------------------------------------------------------
# Variables that may be usefull to change their values

BASENAME           ?= book
SRC_DIR            ?= src
SRC_COVER          ?= $(SRC_DIR)/cover.svg
BLD_DIR            ?= build
MARKDOWN_FORMAT    ?= markdown
TOP_LEVEL_DIVISION ?= chapter

#-----------------------------------------------------------------------------
# Begin of the makefile, internal but usefull variables

all :

clean ::

cleanall :: clean

.PHONY : all clean cleanall

BOOKMAKER_DIR := $(shell dirname $(realpath $(lastword $(MAKEFILE_LIST))))

#------------------------------------------------------------------------------
# fonts
#
# As the fonts used by the project may not be available on the building system,
# those reciepes have been added to install provided fonts (in the FONT_SRC_DIR)
#
# To activate this font installation process, define the FONT_DIR (to the value
# of the directory where you placed your fonts).
#

ifdef FONTS_DIR

FONTS_DST_DIR  = ~/.fonts
FONTS_SRC_FILE = $(shell find $(FONTS_DIR) -type f)
FONTS_DST_FILE = $(subst $(FONTS_DIR),$(FONTS_DST_DIR),$(FONTS_SRC_FILE))

$(FONTS_DST_DIR):
	mkdir -p $@

$(FONTS_DST_DIR)/%: $(FONTS_DIR)/% $(FONTS_DST_DIR)
	cp $< $@

.PHONY: fonts

fonts : $(FONTS_DST_FILE)

clean ::
	rm -rf $(FONTS_DST_FILE)

else

fonts:

endif

#------------------------------------------------------------------------------
# Pandoc
#

#
# The sources files that will be compiled
#
SRC_FILES = $(sort $(shell find $(SRC_DIR) -type f -name "*.md"))
BLD_FILES = $(subst $(SRC_DIR),$(BLD_DIR),$(SRC_FILES))

# To make build cleaner, we first produce an "absolute" and "shifted" version
# of every markdown file :
#
# * Absolute : The links to images and files that are relative to the files
#              directory will be translate as relative to the working
#              directory (so one can then ask pandoc to compile them all)
#
#              This is achieved with the rebase_relative_path extension
#
# * Shifted  : As it is cleaner to have everyfile with first level titles,
#              we shift them to the corresponding directory depth.
#
#              This is achieved with the --shift-heading-level-by argument

$(BLD_DIR)/%.md : $(SRC_DIR)/%.md
	mkdir -p $(dir $@)
	pandoc -s -o $@ \
		--from $(MARKDOWN_FORMAT)-auto_identifiers+rebase_relative_paths \
		--to   $(MARKDOWN_FORMAT)-auto_identifiers \
		--shift-heading-level-by=$$(echo $(subst $(SRC_DIR)/,,$<) | tr -cd '/' |wc -c) \
		$^

clean ::
	rm -rf $(BLD_DIR)

# Produces the _main_ markdown file that include the whole markdown files in
# ABS_DIR. This file is the base point for all subsequent compilations.

PROJECT_MD = $(BASENAME).md

$(PROJECT_MD): $(BLD_FILES)
	pandoc \
		-s -o $@ \
		--from $(MARKDOWN_FORMAT)-auto_identifiers \
		--to   $(MARKDOWN_FORMAT)-auto_identifiers \
		--reference-location block \
		--file-scope \
		$^

clean ::
	rm -f $(PROJECT_MD)

.PHONY: markdown

markdown: $(PROJECT_MD)

#------------------------------------------------------------------------------
# latex
#
# As we compile directly from markdown, this tex file is not mandatory for the
# build process. It is provided as a way to debug strange latex compilations
# errors.

PROJECT_TEX      := $(BASENAME).tex
TEX_TEMPLATE     := $(BOOKMAKER_DIR)/global.tex

$(PROJECT_TEX): $(PROJECT_MD) $(TEX_TEMPLATE)
	pandoc \
		-s -o $@ \
		--from $(MARKDOWN_FORMAT)-auto_identifiers \
		--top-level-division=$(TOP_LEVEL_DIVISION) \
		--template $(TEX_TEMPLATE) \
		$< -o $@

clean ::
	rm -f $(PROJECT_TEX) 

.PHONY: latex

latex: $(PROJECT_TEX)

#------------------------------------------------------------------------------
# png
#
# Compilation of cover png files. As different target needs different dpi, the
# target have been writtent accordingly

PNG_FRONT = front.png
PNG_BACK  = back.png
PNG_COVER = cover.png

PNG_FILES_100 = $(PNG_FRONT) $(PNG_BACK)
PNG_FILES_300 = $(PNG_COVER)
PNG_FILES     = $(PNG_FILES_100) $(PNG_FILES_300) 

$(PNG_FILES_100): $(SRC_COVER) | fonts
	inkscape --export-dpi=100 --export-id=export-$(basename $(notdir $@)) --export-png=$@ $< > /dev/null

$(PNG_FILES_300): $(SRC_COVER) | fonts
	inkscape --export-dpi=300 --export-id=export-$(basename $(notdir $@)) --export-png=$@ $< > /dev/null

clean ::
	rm -rf $(PNG_FILES)
	
.PHONY: png cover

png: $(PNG_FILES)

cover: $(PNG_COVER)

#------------------------------------------------------------------------------
# pdf
#
# Step 1. The text only
#

PROJECT_PDF      = $(BASENAME).pdf
PROJECT_PDF_FULL = $(BASENAME)-full.pdf
PDF_FRONT        = front.pdf
PDF_BACK         = back.pdf

$(PROJECT_PDF): $(PROJECT_MD) $(TEX_TEMPLATE) | fonts
	pandoc \
		-s -o $@ \
		--from $(MARKDOWN_FORMAT)-auto_identifiers \
		--top-level-division=$(TOP_LEVEL_DIVISION) \
		--template $(TEX_TEMPLATE) \
		--pdf-engine=xelatex \
		$<

# Step 2. The covers

%.pdf: %.png
	convert $< $@

$(PROJECT_PDF_FULL) : $(PDF_FRONT) $(PROJECT_PDF) $(PDF_BACK)
	pdftk $^ cat output $@

clean ::
	rm -rf $(PROJECT_PDF) $(PDF_FRONT) $(PDF_BACK) $(PROJECT_PDF_FULL) 

.PHONY: pdf pdf-full

pdf: $(PROJECT_PDF)

pdf-full: $(PROJECT_PDF_FULL)

#------------------------------------------------------------------------------
# EPUB
#
#

PROJECT_EPUB := $(BASENAME).epub

$(PROJECT_EPUB) : $(PROJECT_MD) $(PNG_FRONT)
	pandoc \
		-s -o $@ \
		--from $(MARKDOWN_FORMAT)-auto_identifiers \
		--epub-cover-image=$(PNG_FRONT) \
		$<

clean ::
	rm -f $(PROJECT_EPUB)
	
.PHONY: epub

epub: $(PROJECT_EPUB)

